__author__ = "Fabio Caccamo"
__copyright__ = "Copyright (c) 2020-present Fabio Caccamo"
__description__ = "config and manage typed extra settings using just the django admin."
__email__ = "fabio.caccamo@gmail.com"
__license__ = "MIT"
__title__ = "django-extra-settings"
__version__ = "0.12.0"
