# Generated by Django 3.2.8 on 2022-10-24 07:17

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("extra_settings", "0007_alter_settings_validator"),
    ]

    operations = [
        migrations.AlterField(
            model_name="setting",
            name="name",
            field=models.CharField(
                help_text="(e.g. SETTING_NAME)",
                max_length=255,
                unique=True,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="setting",
            name="value_string",
            field=models.CharField(blank=True, max_length=255, verbose_name="Value"),
        ),
    ]
